import cocos
import pymunk
from pymunk import Vec2d
from cocos.draw import Line
import cocos.cocosnode
from cocos.layer import ColorLayer
import pymunk.pyglet_util
import  random

def resetX(x):
    v = x.body.position.y
    x.update(position = (320, v), rotation_vector=(3.14/4, 0))
    x.body.position = (320, v)

def resetX1(x):
    v = x.body.position.y
    x.update(position = (0, v), rotation_vector=(3.14/4, 0))
    x.body.position = (0, v)

class PhysicsDebugNode(cocos.layer.Layer):
    def __init__(self, space):
        super(PhysicsDebugNode, self).__init__()
        self.space = space

    def draw(self, *args, **kwargs):
        pymunk.pyglet_util.draw(self.space)

class GameOverScreen(cocos.layer.Layer):
    def __init__(self, game):
        super(GameOverScreen, self).__init__()
        w,h = cocos.director.director.get_window_size()
        self.heading = cocos.text.Label("Score: " + str(game.score), anchor_x='center', font_size=30)
        self.heading.position = (w/2, 0.75*h)
        #self.add(self.heading)

        menu = cocos.menu.Menu()
        positions = [(w/2, h/2),]

        self.game = game
        layout_func = cocos.menu.fixedPositionMenuLayout(positions)
        l = []
        l.append(cocos.menu.MenuItem('RESTART', self.on_restart))
        menu.create_menu(l, cocos.menu.zoom_in(), cocos.menu.zoom_out())

        menu.position = 0,0
        menu.scale = 0.25
        menu.on_quit = self.on_restart
        self.add(menu)

    def on_restart(self):
        self.game.reset()
        self.game.remove(self)


class HelloWorld(cocos.layer.Layer):
    HERO = 1
    SPIKES = 2
    LEFT_WALL = 3
    RIGHT_WALL = 4
    is_event_handler = True
    def __init__(self):
        super(HelloWorld, self).__init__()
        ### Physics stuff
        space = pymunk.Space()
        self.space = space
        w,h = cocos.director.director.get_window_size()
        self.left_spikes = []
        self.right_spikes = []
        self.space.gravity = Vec2d(0.0, -400.0)

        self.game_started = False
        self.game_over = False
        self.score = 0
        self.score_label = cocos.text.Label("Score: " + str(self.score), font_size=15, anchor_x='center', anchor_y='top')
        self.score_label.position = w/2,h
        self.add(self.score_label)

        static_body = pymunk.Body()
        self.segments = [ pymunk.Segment(space.static_body, (0, 0), (w, 0), 1)
                ,pymunk.Segment(space.static_body, (w, 0), (w, h), 1)
                ,pymunk.Segment(space.static_body, (w, h), (0, h), 1)
                ,pymunk.Segment(space.static_body, (0, h), (0, 0), 1)
                ]


        self.space.add(self.segments)

        b = pymunk.Body(1, 1)
        b.position = (w/2, h/2)
        s = pymunk.Circle(b, 10)
        s.elasticity = 1
        s.collision_type = 1
        self.space.add(b, s)


        # l = ColorLayer( 0, 255,0,255, width=30, height=30)
        # l.position = (w/2,h/2)
        # self.add(l)

        #self.hbdraw = l
        self.hb = b

        for segment in self.segments:
            segment.elasticity = 1.0
            #self.add(cocos.draw.Line(segment.a, segment.b, (255, 0, 0, 255), 5))


        self.addSpikes()

        #Handle collision with the walls.
        self.segments[1].collision_type = self.RIGHT_WALL
        self.segments[3].collision_type = self.LEFT_WALL
        self.space.add_collision_handler(self.HERO, self.LEFT_WALL, separate=self.leftWallHit)
        self.space.add_collision_handler(self.HERO, self.RIGHT_WALL, separate=self.rightWallHit)

        #add the tip to start.
        self.info = cocos.text.Label("Click to Start", anchor_x='center')
        self.info.position = (160, 150)
        self.add(self.info)

        self.schedule(self.update)

        self.reset()
        self.add(PhysicsDebugNode(self.space))


    def reset(self):
        w,h = cocos.director.director.get_window_size()
        self.score = 0
        self.game_over = False
        self.game_started = False
        self.info.opacity = 255
        self.hb.reset_forces()
        self.hb.position = (w/2, h/2)
        self.updateScoreLabel()
        map(resetX1, self.left_spikes)
        self.leftWallHit(True, True)


    def addSpikes(self):
        for x in range(20, 320, 40):
            brick_body = pymunk.Body()
            brick_body.position = x, 480
            brick_body.angle = 3.14/4;
            brick_shape = pymunk.Poly.create_box(brick_body, (30,30))
            brick_shape.elasticity = 0.5
            brick_shape.color = (0, 0, 255)
            brick_shape.group = 1
            brick_shape.collision_type = 2
            self.space.add(brick_shape)

            brick_body = pymunk.Body()
            brick_body.position = x, 0
            brick_body.angle = 3.14/4;
            brick_shape = pymunk.Poly.create_box(brick_body, (30,30))
            brick_shape.elasticity = 0.5
            brick_shape.color = (0, 0, 255)
            brick_shape.group = 1
            brick_shape.collision_type = 2
            self.space.add(brick_shape)

        for y in range(20, 480, 40):
            brick_body = pymunk.Body()
            brick_body.position = 0, y
            brick_body.angle = 3.14/4;
            brick_shape = pymunk.Poly.create_box(brick_body, (30,30))
            brick_shape.elasticity = 0.1
            brick_shape.color = (0, 0, 255)
            brick_shape.group = 1
            brick_shape.collision_type = 2
            self.space.add(brick_shape)
            self.left_spikes.append(brick_shape);

            brick_body = pymunk.Body()
            brick_body.position = 320, y
            brick_body.angle = 3.14/4;
            brick_shape = pymunk.Poly.create_box(brick_body, (30,30))
            brick_shape.elasticity = 0.1
            brick_shape.color = (0, 0, 255)
            brick_shape.group = 1
            brick_shape.collision_type = 2
            self.space.add(brick_shape)
            self.right_spikes.append(brick_shape);

        self.space.add_collision_handler(1, 2, begin=self.test)

    def updateScoreLabel(self):
        self.score_label.element.text = "Score: " + str(self.score)

    def leftWallHit(self, space, arbiter):
        if self.game_over:
            return
        if self.game_started:
            self.score += 1
            self.updateScoreLabel()
        map(resetX, self.right_spikes)
        random.shuffle(self.right_spikes)

        range_val = min(1+self.score, len(self.right_spikes)-4)
        range_val = len(self.right_spikes) - range_val

        for i in range(range_val):
            prev_y = self.right_spikes[i].body.position.y
            self.right_spikes[i].update(position = (360,prev_y), rotation_vector=(3.14/4, 0))
            self.right_spikes[i].body.position = (360, prev_y)
        pass

    def rightWallHit(self, space, arbiter):
        if self.game_over:
            return
        if self.game_started:
            self.score += 1
            self.updateScoreLabel()
        map(resetX1, self.left_spikes)
        random.shuffle(self.left_spikes)

        range_val = min(1+self.score, len(self.right_spikes)-4)
        range_val = len(self.right_spikes) - range_val
        for i in range(range_val):
            prev_y = self.left_spikes[i].body.position.y
            self.left_spikes[i].update(position = (-40,prev_y), rotation_vector=(3.14/4, 0))
            self.left_spikes[i].body.position = (-40, prev_y)
        pass

    def test(self, space, arbiter):
        #game over
        if self.game_over == False:
            gmover = GameOverScreen(self)
            self.add(gmover)

        self.game_over = True
        return True

    def update(self, dt):
        if self.game_started:
            sdt = 1.0/60. #override dt to keep physics simulation stable
            self.space.step(sdt)
        #print self.hb.position
        #self.hbdraw.position = (self.hb.position.x-self.hbdraw.width/2, self.hb.position.y-self.hbdraw.height/2)

    def on_mouse_press(self, x, y, buttons, modifiers):
        if self.game_over:
            return
        if self.game_started:
            self.hb.apply_impulse(Vec2d(0, 250), Vec2d(0,0))
        else:
            self.info.do(cocos.actions.FadeOut(1));
            self.hb.apply_impulse(Vec2d(200,250), Vec2d(0,0))
            self.game_started = True
